# -*- python -*-
# $RCSfile: tsl.py,v $
# $Revision: 1.7 $
# $Author: langer $
# $Date: 2007-05-31 19:00:03 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.SWIG.common import ooferror
from oof2.SWIG.engine import corientation
from oof2.SWIG.orientationmap import orientmapdata
from oof2.common import debug
from oof2.common import primitives
from oof2.common import utils

import math

class DataPoint:
    def __init__(self, position, angletuple, phasename):
        self.position = position
        radians = math.pi/180.
        self.angle = corientation.COrientBunge(*[a*radians for a in angletuple])
        self.phasename = phasename
    def euler(self):
        return self.angle.abg()

class TSLreader(orientmapdata.OrientMapReader):
    def __init__(self):
        self.phaselists = {}
        orientmapdata.OrientMapReader.__init__(self)
    def read(self, filename):
        tslfile = file(filename, "r")
        count = 1                       # line counter
        lines = tslfile.readlines()
        nlines = len(lines)
        data = utils.ReservableList(nlines)
        for line in lines:
            if line[0] != '#':
                substrings = line.split()
                if len(substrings) < 5:
                    raise ooferror.ErrUserError(
                        "Too few numbers in line %d of %s" % (count, filename))
                values = map(float, substrings[:5])
                data.append(DataPoint(
                    primitives.Point(values[3], values[4]), # position
                    (values[0], values[1], values[2]), # angle
                    substrings[10]))    # phase name
            count += 1      # count actual file lines, comments and all
        npts = len(data)
        debug.fmsg("read %d lines, %d data points" % (count, npts))
        # find range of x and y values, and number of each.
        x0 = data[0].position[0]
        y0 = data[1].position[1]
        for i in range(1, npts):
            if data[i].position[0] == x0:
                width = data[i-1].position[0]
                nx = i                  # number of points in x direction
                ny = npts/nx            # number of points in y direction
                if nx*ny != npts:
                    raise ooferror.ErrUserError(
                        "Orientation map data appears to be incomplete.")
                break
        else:                           # only one line of data? 
            raise ooferror.ErrUserError(
                "Orientation map data appears to be incomplete.")
    
        # pixel size
        pxlsize = primitives.Point(data[1].position[0] - data[0].position[0],
                                   data[nx].position[1] - data[0].position[1])
        # If we assume that the points are in the centers of the
        # pixels, then the actual physical size is one pixel bigger
        # than the range of the xy values.
        size = data[-1].position + pxlsize
        
        od = orientmapdata.OrientMap(primitives.iPoint(nx, ny), size)

        for datum in data:
            ij = primitives.iPoint(
                int(round(datum.position[0]/pxlsize[0])),
                ny-1-int(round(datum.position[1]/pxlsize[1])))
            try:
                self.phaselists[datum.phasename].append(ij)
            except KeyError:
                self.phaselists[datum.phasename] = [ij]
            self.set_angle(od, ij, datum.euler())
        return od

    ## postProcess is called after the orientation data has been
    ## assigned to a Microstructure.
    def postProcess(self, microstructure):
        phasenames = self.phaselists.keys()
        phasenames.sort()
        for phasename in phasenames:
            orientmapdata.addPixelsToGroup(microstructure, phasename,
                                           self.phaselists[phasename])

orientmapdata.registerOrientMapReaderClass('TSL', TSLreader,
                                           help='TSL .ang file')
        
